`timescale 1ns / 1ps
//////////////////////////////////////////////////////////////////////////////////
// Company: 
// Engineer: 
// 
// Create Date:    23:54:21 11/17/2013 
// Design Name: 
// Module Name:    main 
// Project Name: 
// Target Devices: 
// Tool versions: 
// Description: 
//
// Dependencies: 
//
// Revision: 
// Revision 0.01 - File Created
// Additional Comments: 
//
//////////////////////////////////////////////////////////////////////////////////

module UltiMem(reset, clock, r_w, address, data, cart_en, io, flash_ce, ram_ce, we, oe, ram1,ram2,ram3, blk1, blk2, blk3, blk5, bank);
inout reset;
input clock;
input r_w;
input [9:0]address;
inout [7:0]data;
input cart_en;
input io;
//input io_sel;
output flash_ce;
output ram_ce;
output we;
output oe;
input ram1;
input ram2;
input ram3;
input blk1;
input blk2;
input blk3;
input blk5;
output [6:0]bank;

wire is_addr_valid;
wire ram_ram_en;
wire blk1_ram_en;
wire blk2_ram_en;
wire blk3_ram_en;
wire blk5_ram_en;

wire ram_rom_en;
wire blk1_rom_en;
wire blk2_rom_en;
wire blk3_rom_en;
wire blk5_rom_en;

wire [2:0] cart_config1;
wire [7:0] cart_config2;
wire [6:0] ram_bank;
wire [6:0] blk1_bank;
wire [6:0] blk2_bank;
wire [6:0] blk3_bank;
wire [6:0] blk5_bank;
wire [7:0] reg_data;
wire [6:0] blk_bank;
wire [1:0] blk_address;
wire [7:0] temp_data;

// assigns
assign ram_sel = (!ram1 | !ram2 | !ram3);	// active high

// 00 off
// 01	rom
// 10	ram
// 11 ram no write

assign ram_rom_en  = (cart_config1[1:0] == 01);
assign blk1_rom_en = (cart_config2[1:0] == 01);
assign blk2_rom_en = (cart_config2[3:2] == 01);
assign blk3_rom_en = (cart_config2[5:4] == 01);
assign blk5_rom_en = (cart_config2[7:6] == 01);

assign ram_ram_en  = (cart_config1[1] == 1);
assign blk1_ram_en = (cart_config2[1] == 1);
assign blk2_ram_en = (cart_config2[3] == 1);
assign blk3_ram_en = (cart_config2[5] == 1);
assign blk5_ram_en = (cart_config2[7] == 1);


assign flash_ce = !(((ram_sel & ram_rom_en) | (!blk1 & blk1_rom_en) | (!blk2 & blk2_rom_en) | (!blk3 & blk3_rom_en) | (!blk5 & blk5_rom_en))); //active low

assign ram_ce = !(((ram_sel & ram_ram_en) | (!blk1 & blk1_ram_en) | (!blk2 & blk2_ram_en) | (!blk3 & blk3_ram_en) | (!blk5 & blk5_ram_en))); //active low

assign we = !(!r_w & cart_en & (cart_config1[0] | cart_config2[0] | cart_config2[2] | cart_config2[4] | cart_config2[6])); // active low
assign oe = !(r_w & cart_en);  // active low

assign reg_ce = (!cart_config1[2] & cart_en & !io & is_addr_valid); // active high

assign reg_write = (!r_w & reg_ce);

assign cart_config1_reg_ce = 	(reg_write & (address[2:0] == 0)); //active high
assign cart_config2_reg_ce = 	(reg_write & (address[2:0] == 1)); //active high
//assign temp_reg_ce = 			(reg_write & (address[2:0] == 2)); //active high
assign ram_bank_reg_ce = 		(reg_write & (address[2:0] == 3)); //active high
assign blk1_bank_reg_ce = 		(reg_write & (address[2:0] == 4)); //active high
assign blk2_bank_reg_ce = 		(reg_write & (address[2:0] == 5)); //active high
assign blk3_bank_reg_ce = 		(reg_write & (address[2:0] == 6)); //active high
assign blk5_bank_reg_ce = 		(reg_write & (address[2:0] == 7)); //active high

assign data = (r_w & reg_ce & clock ? reg_data : 8'bz);

//wire reset_en;
//assign reset_en = (cart_config1_reg_ce & !clock & data[7]);
reg reset_en = 0;
always @(negedge clock) reset_en <= (cart_config1_reg_ce & data[7]);

assign reset    = (reset_en ? 0 : 1'bz);
assign reset_in = (reset_en ? 0 : !reset);

// instances
register #(.WIDTH(3))	cart_config1_reg(!clock, reset_in, cart_config1_reg_ce, data[2:0], cart_config1);
register #(.RESET(1))	cart_config2_reg(!clock, reset_in, cart_config2_reg_ce, data, cart_config2);
//register 					cart_config2_reg(!clock, !reset, cart_config2_reg_ce, data, cart_config2);
//register 					temp_reg(!clock, reset_in, temp_reg_ce, data[7:0], temp_data);
assign temp_data = 0;
register #(.WIDTH(7))	ram_bank_reg(!clock, reset_in, ram_bank_reg_ce, data[6:0], ram_bank);
register #(.WIDTH(7))	blk1_bank_reg(!clock, reset_in, blk1_bank_reg_ce, data[6:0], blk1_bank);
register #(.WIDTH(7))	blk2_bank_reg(!clock, reset_in, blk2_bank_reg_ce, data[6:0], blk2_bank);
register #(.WIDTH(7))	blk3_bank_reg(!clock, reset_in, blk3_bank_reg_ce, data[6:0], blk3_bank);
register #(.WIDTH(7))	blk5_bank_reg(!clock, reset_in, blk5_bank_reg_ce, data[6:0], blk5_bank);

comparator #(.WIDTH(7))	address_comp(address[9:3], 7'b1, is_addr_valid);

mux8_1 					reg_mux(address[2:0],{6'b0,cart_config1[1:0]},cart_config2,temp_data,{0,ram_bank}, {0,blk1_bank}, {0,blk2_bank}, {0,blk3_bank}, {0,blk5_bank}, reg_data);
// OR
//mux8_1 #(.WIDTH(7))   	reg_mux(address[2:0],{5'b0,cart_config1},cart_config2[6:0],0,ram_bank, blk1_bank, blk2_bank, blk3_bank, blk5_bank, reg_data[6:0]);
//assign reg_data[7] = ((address[2:0] == 1) ? cart_config2[7] : 0);

//mux8_1 #(.WIDTH(7))     bank_mux({ram_sel,blk_address},blk1_bank, blk2_bank, blk3_bank, blk5_bank, ram_bank, ram_bank, ram_bank, ram_bank, bank);
// OR
mux4_1 #(.WIDTH(7))     blk_bank_mux(blk_address,blk1_bank, blk2_bank, blk3_bank,blk5_bank,blk_bank); 
mux2_1 #(.WIDTH(7))    	bank_mux(ram_sel,blk_bank, ram_bank, bank);

encoder4_2					blk_encoder(!blk1,!blk2,!blk3,!blk5,blk_address);

endmodule

//`include "comparator.v"
//`include "register.v"
//`include "mux.v"
//`include "encoder.v"


